/*
 * albe_orig.h - albe potential header file
 *
 * author: Frank Zirkelbach <frank.zirkelbach@physik.uni-augsburg.de>
 *
 */

#ifndef ALBE_ORIG_H
#define ALBE_ORIG_H

/* albe constants */
#include "albe.h"

#define ALBE_ORIG_MAXN	16*27

/* albe exchange type */
typedef struct s_albe_orig_exchange {

	t_3dvec dist_ij;
	double d_ij2;
	double d_ij;

	t_3dvec dist_ik[ALBE_ORIG_MAXN];
	double d_ik2[ALBE_ORIG_MAXN];
	double d_ik[ALBE_ORIG_MAXN];

	double f_c_ik[ALBE_ORIG_MAXN];
	double df_c_ik[ALBE_ORIG_MAXN];

	double g[ALBE_ORIG_MAXN];
	double dg[ALBE_ORIG_MAXN];
	double cos_theta[ALBE_ORIG_MAXN];

	double *gamma_i;
	double *c_i;
	double *d_i;
	double *h_i;

	double *ci2;
	double *di2;
	double *ci2di2;

	double zeta_ij;
	double pre_dzeta;

	int kcount;
} t_albe_orig_exchange;

/* albe mult (2!) potential parameters */
typedef struct s_albe_orig_mult_params {
	double S[2];		/* albe cutoff radii */
	double S2[2];		/* albe cutoff radii squared */
	double R[2];		/* albe cutoff radii */
	double Smixed;		/* mixed S radius */
	double S2mixed;		/* mixed S radius squared */
	double Rmixed;		/* mixed R radius */
	double A[2];		/* factor of albe attractive part */
	double B[2];		/* factor of albe repulsive part */
	double r0[2];		/* r_0 */
	double Amixed;		/* mixed A factor */
	double Bmixed;		/* mixed B factor */
	double r0_mixed;	/* mixed r_0 */
	double lambda[2];	/* albe lambda */
	double lambda_m;	/* mixed lambda */
	double mu[2];		/* albe mu */
	double mu_m;		/* mixed mu */

	double gamma[2];
	double gamma_m;
	double c[2];
	double c2[2];
	double c_mixed;
	double c2_m;
	double d[2];
	double d2[2];
	double d_mixed;
	double d2_m;
	double h[2];
	double h_mixed;
	double c2d2[2];
	double c2d2_m;

	t_albe_orig_exchange exchange;	/* exchange between 2bp and 3bp calc */
} t_albe_orig_mult_params;

/* function prototypes */
int albe_orig_mult_set_params(t_moldyn *moldyn,int element1,int elemnt2);
int albe_orig_mult_3bp_j1(t_moldyn *moldyn,t_atom *ai,t_atom *aj,u8 bc);
int albe_orig_mult_3bp_k1(t_moldyn *moldyn,
                          t_atom *ai,t_atom *aj,t_atom *ak,u8 bc);
int albe_orig_mult_3bp_j2(t_moldyn *moldyn,t_atom *ai,t_atom *aj,u8 bc);
int albe_orig_mult_3bp_k2(t_moldyn *moldyn,
                          t_atom *ai,t_atom *aj,t_atom *ak,u8 bc);
int albe_orig_mult_check_2b_bond(t_moldyn *moldyn,
                                 t_atom *itom,t_atom *jtom,u8 bc);

/* albe potential parameter defines */
// -> albe.h

#endif
